/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.atlas.hdfs.bridge;

import org.apache.atlas.exception.AtlasBaseException;
import org.apache.atlas.hdfs.lineage.Constants;
import org.apache.atlas.hook.AtlasHook;
import org.apache.atlas.model.instance.AtlasEntity;
import org.apache.atlas.model.instance.AtlasObjectId;
import org.apache.atlas.model.notification.HookNotification;
import org.apache.atlas.type.AtlasType;
import org.apache.commons.collections.CollectionUtils;
import org.apache.hadoop.security.UserGroupInformation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.Collections;
import java.util.List;

public class AtlasHDFSHook extends AtlasHook {
    private static final Logger LOG             = LoggerFactory.getLogger(AtlasHDFSHook.class);
    private static final AtlasHDFSHook INSTANCE = new AtlasHDFSHook();
    public static final String HDFS_SOURCE       = "hdfs";

    private UserGroupInformation ugi;

    public static AtlasHDFSHook getInstance() {
        return INSTANCE;
    }

    private AtlasHDFSHook() {
    }

    public String getMessageSource() {
        return HDFS_SOURCE;
    }

    public void initialize() throws AtlasBaseException {
        try {
            ugi = UserGroupInformation.getCurrentUser();
        } catch (IOException e) {
            String msg = "Error encountered in getting the user information.";

            LOG.error(msg, e);

            throw new AtlasBaseException(msg, e);
        }

        LOG.info("Initialization completed successfully.");
    }

    public void publishEvent(AtlasHDFSEvent event) throws Exception {
        switch (event.getEventType()) {
            case CREATE:
                publishCreateEventNotification(event);
                break;

            case DELETE:
                publishDeleteEventNotification(event);
                break;

            default:
                throw new AtlasBaseException("Unknown event type received. EventType: " + event.getEventType());
        }
    }

    private void publishDeleteEventNotification(AtlasHDFSEvent event){

        if (event instanceof AtlasHDFSEvent) {
            AtlasHDFSDeleteEvent deleteEvent = (AtlasHDFSDeleteEvent) event;

            if (LOG.isDebugEnabled()) {
                LOG.debug("Sending delete event to Atlas: {}", deleteEvent.getEntiObjectIds().toString());
            }

            List<HookNotification>   notificationMessages = null;
            List<AtlasObjectId> atlasObjectIds  = deleteEvent.getEntiObjectIds();

            if (atlasObjectIds != null && CollectionUtils.isNotEmpty(atlasObjectIds)) {
                notificationMessages = Collections.singletonList(new HookNotification.EntityDeleteRequestV2(deleteEvent.getUserName(), atlasObjectIds));
            }

            super.notifyEntities(notificationMessages, ugi);

            if (LOG.isDebugEnabled()) {
                LOG.debug("Message sent is {}", AtlasType.toJson(notificationMessages));
            }
        }
    }

    private void publishCreateEventNotification(AtlasHDFSEvent event) {
        if (event instanceof AtlasHDFSEvent) {
            AtlasHDFSCreateEvent createEvent = (AtlasHDFSCreateEvent) event;

            if (LOG.isDebugEnabled()) {
                LOG.debug("Sending create event to Atlas: {}", createEvent.getEntitiesWithExtInfo().toString());
            }

            List<HookNotification>   notificationMessages = null;
            AtlasEntity.AtlasEntitiesWithExtInfo entitiesWithExtInfo  = createEvent.getEntitiesWithExtInfo();

            if (entitiesWithExtInfo != null && CollectionUtils.isNotEmpty(entitiesWithExtInfo.getEntities())) {
                notificationMessages = Collections.singletonList(new HookNotification.EntityCreateRequestV2(createEvent.getUserName(), entitiesWithExtInfo));
            }

            super.notifyEntities(notificationMessages, ugi);

            if (LOG.isDebugEnabled()) {
                LOG.debug("Message sent is {}", AtlasType.toJson(notificationMessages));
            }
        }
    }

    public String getJAASKeyTab(){
        return atlasProperties.getString(Constants.JAAS_KEYTAB, "");
    }


    public String getJAASPrincipal(){
        return atlasProperties.getString(Constants.JAAS_PRINCIPAL, "");
    }
}
