/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.atlas.hdfs.lineage;

import org.apache.atlas.exception.AtlasBaseException;
import org.apache.atlas.hdfs.bridge.AtlasHDFSHook;
import org.apache.atlas.hdfs.cli.CliContext;
import org.apache.commons.lang.StringUtils;
import org.springframework.util.ObjectUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.net.URI;
import java.util.*;

/**
 * A context class that contains shared information for lineage creation
 */
public class HDFSLineageContext implements Serializable {

    private static final long   serialVersionUID = 1L;
    private static final Logger LOG              = LoggerFactory.getLogger(HDFSLineageContext.class);


    private final Set<String>   whitelistingBucketAndKey;
    private final Set<String>   blacklistingBucketAndKey;
    private final AtlasHDFSHook atlasHDFSHook;

    protected final Properties lineageConfiguration;

    public HDFSLineageContext(CliContext cliContext) throws AtlasBaseException {
        File configFile = cliContext.getConfigFile();

        this.atlasHDFSHook = AtlasHDFSHook.getInstance();
        this.lineageConfiguration = loadConfigFile(configFile);
        this.whitelistingBucketAndKey = new TreeSet<>();
        this.blacklistingBucketAndKey = new TreeSet<>();
    }

    public void initialize() throws AtlasBaseException {
        validateConfiguration();

        atlasHDFSHook.initialize();

        validateAtlasClientProperties();

        loadWhitelisting();

        loadBlacklisting();
    }

    public String getNameNodeURIConfig() {
        return lineageConfiguration.getProperty(Constants.CONFIG_HDFS_NAME_NODE_URI);
    }

    public String getWhitelistPaths() {
        return lineageConfiguration.getProperty(Constants.CONFIG_WHITELIST_PATHS);

    }

    protected void validateConfiguration() throws AtlasBaseException {
        checkEmptinessForConfigurationParameter(Constants.CONFIG_HDFS_NAME_NODE_URI, getNameNodeURIConfig());
        checkEmptinessForConfigurationParameter(Constants.CONFIG_WHITELIST_PATHS, getWhitelistPaths());

    }

    protected void validateAtlasClientProperties() throws AtlasBaseException {
        checkEmptinessForConfigurationParameter(Constants.JAAS_KEYTAB, atlasHDFSHook.getJAASKeyTab());
        checkEmptinessForConfigurationParameter(Constants.JAAS_PRINCIPAL, atlasHDFSHook.getJAASPrincipal());

    }

    protected void checkEmptinessForConfigurationParameter(String configurationParameter, Object configuredValue) throws AtlasBaseException {
        if (ObjectUtils.isEmpty(configuredValue)) {
            String msg = generateMissingConfigurationMessage(configurationParameter);

            LOG.error(msg);

            throw new AtlasBaseException(msg);
        }
    }

    protected String generateMissingConfigurationMessage(String lineageConfigurationName) {
        return String.format("There is no value specified for configuration parameter %s", lineageConfigurationName);
    }


    boolean doesPathNeedExtraction(String path) {
        boolean ret = false;

        String pathKey = path.replace(getNameNodeURIConfig(), "");
        if (!isWhiteListingConfigurationPresent()) {
            ret = true;
        } else {
            //Check white listing first
            for (String keyFromList : whitelistingBucketAndKey) {
                if (pathKey.indexOf(keyFromList) == 0 || keyFromList.indexOf(pathKey) == 0) {
                    ret = true;
                    break;
                }
            }
        }

        //Now check if it is also black listed
        for (String keyFromList : blacklistingBucketAndKey) {
            if (pathKey.indexOf(keyFromList) == 0) {
                ret = false;
                break;
            }
        }


        return ret;
    }

    boolean isWhiteListingConfigurationPresent() {
        return StringUtils.isNotEmpty(getWhitelistPaths());
    }

    private void loadWhitelisting() {
        if (isWhiteListingConfigurationPresent()) {
            for (String eachPathKey : StringUtils.split(getWhitelistPaths(), Constants.CONFIG_PATH_SEPARATOR)) {
                if (StringUtils.isNotBlank(eachPathKey)) {
                    eachPathKey = getPathWithoutScheme(eachPathKey);

                    if (whitelistingBucketAndKey.contains(eachPathKey)) {
                        if (LOG.isDebugEnabled()) {
                            LOG.debug("The white list{} is already present ", eachPathKey);
                        }
                    } else {
                        whitelistingBucketAndKey.add(eachPathKey);

                        if (LOG.isDebugEnabled()) {
                            LOG.debug("Adding path{} to white list", eachPathKey);
                        }
                    }
                }
            }

            squeezeKeyEntries(whitelistingBucketAndKey);
        }
    }

    private void loadBlacklisting() {
        String configBlackListing = lineageConfiguration.getProperty(Constants.CONFIG_BLACKLIST_PATHS);

        if (StringUtils.isNotEmpty(configBlackListing)) {
            for (String eachPathKey : StringUtils.split(configBlackListing, Constants.CONFIG_PATH_SEPARATOR)) {
                if (StringUtils.isNotBlank(eachPathKey)) {
                    boolean doesNeedToAddToBlackList = true;
                    eachPathKey = getPathWithoutScheme(eachPathKey);

                    if (whitelistingBucketAndKey.contains(eachPathKey)) {
                        removeAllKeysStartsWithBlacklistKey(whitelistingBucketAndKey, eachPathKey);

                        if (whitelistingBucketAndKey.isEmpty()) {
                            if (LOG.isDebugEnabled()) {
                                LOG.debug("The whitelist is empty due to blacklist path{}", eachPathKey);
                            }

                            doesNeedToAddToBlackList = false;
                        }
                    }

                    if (doesNeedToAddToBlackList) {
                        if (blacklistingBucketAndKey.contains(eachPathKey)) {
                            if (LOG.isDebugEnabled()) {
                                LOG.debug("The path{} is already present at blacklist", eachPathKey);
                            }
                        } else {
                            blacklistingBucketAndKey.add(eachPathKey);

                            if (LOG.isDebugEnabled()) {
                                LOG.debug("Adding path{} to black list", eachPathKey);
                            }
                        }
                    }
                }
            }

            squeezeKeyEntries(blacklistingBucketAndKey);
        }
    }

    private void squeezeKeyEntries(Set<String> keys) {
        Iterator<String> itr = keys.iterator();

        String lastParentKey = null;
        while (itr.hasNext()) {
            String currKey = itr.next();
            if (lastParentKey != null && currKey.indexOf(lastParentKey) == 0) {
                if (LOG.isDebugEnabled()) {
                    LOG.debug("Removing key " + currKey + " from whitelist");
                }

                itr.remove();
            } else {
                lastParentKey = currKey;
            }
        }
    }

    private String getPathWithoutScheme(String path) {
        String pathWithoutScheme = StringUtils.trim(path);
        String fileSystemSchemeWithSeparator = Constants.NAME_NODE_URI_PREFIX;
        int indexOfFileSystemSchemeWithSeparator = pathWithoutScheme.indexOf(fileSystemSchemeWithSeparator);

        if (indexOfFileSystemSchemeWithSeparator == 0) {
            pathWithoutScheme = pathWithoutScheme.substring(indexOfFileSystemSchemeWithSeparator + fileSystemSchemeWithSeparator.length());
        }

        return pathWithoutScheme;
    }

    private void removeAllKeysStartsWithBlacklistKey(Set<String> whiteListSet, String blacklistKey) {
        Iterator<String> itr = whiteListSet.iterator();

        while (itr.hasNext()) {
            String eachWhiteList = itr.next();

            if (eachWhiteList.indexOf(blacklistKey) == 0) {
                if (LOG.isDebugEnabled()) {
                    LOG.debug("Removing key " + eachWhiteList + " from whitelist");
                }

                itr.remove();
            }
        }
    }

    private Properties loadConfigFile(File configFile) throws AtlasBaseException {
        LOG.info("Loading {} from {}", configFile.getName(), configFile.getParent());
        Properties configuration = new Properties();

        try (InputStream inputStream = new FileInputStream(configFile);) {
            configuration.load(inputStream);
        } catch (Exception e) {
            String msg = String.format("Error encountered in reading the configuration file %s. Reason: %s", configFile, e.getMessage());

            LOG.error(msg, e);

            throw new AtlasBaseException(msg);
        }

        return configuration;
    }

    public URI getNameNodeURI() {
        return URI.create(getNameNodeURIConfig());
    }
}
